@@ -1,5 +1,5 @@
 package Mojolicious::Plugin::JSONAPI;
-$Mojolicious::Plugin::JSONAPI::VERSION = '1.3';
+$Mojolicious::Plugin::JSONAPI::VERSION = '1.4';
 use Mojo::Base 'Mojolicious::Plugin';
 
 use JSONAPI::Document;
@@ -9,7 +9,7 @@ use Lingua::EN::Inflexion ();
 # ABSTRACT: Mojolicious Plugin for building JSON API compliant applications.
 
 sub register {
-    my ( $self, $app, $args ) = @_;
+    my ($self, $app, $args) = @_;
     $args ||= {};
 
     my %jsonapi_args;
@@ -17,6 +17,7 @@ sub register {
         $jsonapi_args{kebab_case_attrs} = $args->{kebab_case_attrs};
     }
     if (defined($args->{namespace})) {
+
         # It's not really a JSONAPI::Document arg, but it's as close as
         # we'll get to defining a base URL at startup time.
         $jsonapi_args{namespace} = $args->{namespace};
@@ -30,72 +31,72 @@ sub register {
     $jsonapi_args{data_dir} = $args->{data_dir};
 
     # Detect application/vnd.api+json content type, fallback to application/json
-    $app->types->type(
-        json => [ 'application/vnd.api+json', 'application/json' ] );
+    $app->types->type(json => ['application/vnd.api+json', 'application/json']);
 
-    $self->create_route_helpers( $app, $args->{namespace} );
-    $self->create_data_helpers($app, \%jsonapi_args);
-    $self->create_request_helpers($app);
+    $self->create_route_helpers($app, $args->{namespace});
+    $self->create_data_helpers($app,    {%jsonapi_args});
+    $self->create_request_helpers($app, {%jsonapi_args});
     $self->create_error_helpers($app);
 }
 
 sub create_route_helpers {
-    my ( $self, $app, $namespace ) = @_;
+    my ($self, $app, $namespace) = @_;
 
     my $DEV_MODE = $app->mode eq 'development';
 
     $app->helper(
         resource_routes => sub {
-            my ( $c, $spec ) = @_;
+            my ($c, $spec) = @_;
             $spec->{resource} || Carp::confess('resource is a required param');
             $spec->{relationships} ||= [];
             my @DEV_LOGS;
 
-            my $resource = Lingua::EN::Inflexion::noun( $spec->{resource} );
+            my $resource          = Lingua::EN::Inflexion::noun($spec->{resource});
             my $resource_singular = $resource->singular;
             my $resource_plural   = $resource->plural;
 
             my $action_singular = $resource->singular;
             my $action_plural   = $resource->plural;
-            $_ =~ s/-/_/g for ( $action_singular, $action_plural );
+            $_ =~ s/-/_/g for ($action_singular, $action_plural);
 
             my $base_path = (!$spec->{router} && $namespace) ? "/$namespace/$resource_plural" : "/$resource_plural";
             my $router = $spec->{router} ? $spec->{router} : $app->routes;
             my $controller = $spec->{controller} || "api-$action_plural";
 
-            my $r = $router->any($base_path)->to( controller => $controller );
-            $r->get('/')->to( action => "fetch_${action_plural}" );
-            $r->post('/')->to( action => "post_${action_singular}" );
+            my $r = $router->any($base_path)->to(controller => $controller);
+            $r->get('/')->to(action => "fetch_${action_plural}");
+            $r->post('/')->to(action => "post_${action_singular}");
             push @DEV_LOGS, "GET $base_path/ -> ${controller}#fetch_${action_plural}";
             push @DEV_LOGS, "POST $base_path/ -> ${controller}#post_${action_singular}";
             foreach my $method (qw/get patch delete/) {
-                push @DEV_LOGS, uc($method) . " $base_path/:${action_singular}_id -> ${controller}#${method}_${action_singular}";
-                $r->$method("/:${action_singular}_id")->to( action => "${method}_${action_singular}" );
+                push @DEV_LOGS,
+                    uc($method) . " $base_path/:${action_singular}_id -> ${controller}#${method}_${action_singular}";
+                $r->$method("/:${action_singular}_id")->to(action => "${method}_${action_singular}");
             }
 
-            foreach my $relationship ( @{ $spec->{relationships} } ) {
-                my $path ="/:${action_singular}_id/relationships/${relationship}";
+            foreach my $relationship (@{ $spec->{relationships} }) {
+                my $path                = "/:${action_singular}_id/relationships/${relationship}";
                 my $relationship_action = $relationship;
                 $relationship_action =~ s/-/_/g;
                 foreach my $method (qw/get post patch delete/) {
-                    push @DEV_LOGS, uc($method) . " ${base_path}${path} -> ${controller}#${method}_related_${relationship_action}";
-                    $r->$method($path)->to(action => "${method}_related_${relationship_action}" );
+                    push @DEV_LOGS,
+                        uc($method) . " ${base_path}${path} -> ${controller}#${method}_related_${relationship_action}";
+                    $r->$method($path)->to(action => "${method}_related_${relationship_action}");
                 }
             }
 
-            if ( $DEV_MODE ) {
+            if ($DEV_MODE) {
                 $app->log->debug('Created the following JSONAPI routes:');
-                $app->log->debug($_) for @DEV_LOGS;
+                $app->log->debug("\n\t" . join("\n\t", @DEV_LOGS));
             }
-        }
-    );
+        });
 }
 
 sub create_data_helpers {
-    my ( $self, $app, $args ) = @_;
+    my ($self, $app, $args) = @_;
 
-    my $namespace = delete $args->{namespace} // '';
-    my $api_path = $namespace ? '/' . $namespace : '/';
+    my $namespace  = delete $args->{namespace} // '';
+    my $api_path   = $namespace ? '/' . $namespace : '/';
     my $jsonapi_cb = sub {
         my ($c) = @_;
         my $api_url = $c->url_for($api_path)->to_abs;
@@ -108,56 +109,49 @@ sub create_data_helpers {
 
     $app->helper(
         resource_document => sub {
-            my ( $c, $row, $options ) = @_;
-            return $jsonapi_cb->($c)->resource_document( $row, $options );
-        }
-    );
+            my ($c, $row, $options) = @_;
+            return $jsonapi_cb->($c)->resource_document($row, $options);
+        });
 
     $app->helper(
         compound_resource_document => sub {
-            my ( $c, $row, $options ) = @_;
-            return $jsonapi_cb->($c)->compound_resource_document( $row, $options );
-        }
-    );
+            my ($c, $row, $options) = @_;
+            return $jsonapi_cb->($c)->compound_resource_document($row, $options);
+        });
 
     $app->helper(
         resource_documents => sub {
-            my ( $c, $resultset, $options ) = @_;
-            return $jsonapi_cb->($c)->resource_documents( $resultset, $options );
-        }
-    );
+            my ($c, $resultset, $options) = @_;
+            return $jsonapi_cb->($c)->resource_documents($resultset, $options);
+        });
 }
 
 sub create_error_helpers {
-    my ( $self, $app ) = @_;
+    my ($self, $app) = @_;
 
     $app->helper(
         render_error => sub {
-            my ( $c, $status, $errors, $data, $meta ) = @_;
+            my ($c, $status, $errors, $data, $meta) = @_;
 
-            unless ( defined($errors) && ref($errors) eq 'ARRAY' ) {
-                $errors = [
-                    {
+            unless (defined($errors) && ref($errors) eq 'ARRAY') {
+                $errors = [{
                         status => $status || 500,
                         title => 'Error processing request',
-                    }
-                ];
+                    }];
             }
 
             return $c->render(
                 status => $status || 500,
                 json => {
-                    $data ? %$data : (),
-                    $meta ? ( meta => $meta ) : (),
-                    errors => $errors,
-                }
-            );
-        }
-    );
+                    $data ? %$data : (), $meta ? (meta => $meta) : (), errors => $errors,
+                });
+        });
 }
 
 sub create_request_helpers {
-    my ( $self, $app ) = @_;
+    my ($self, $app, $args) = @_;
+
+    my $namespace = $args->{namespace} // '';
 
     $app->helper(
         requested_resources => sub {
@@ -166,8 +160,41 @@ sub create_request_helpers {
             $param =~ s/-/_/g;
             my @include = split(',', $param);
             return \@include;
-        }
-    );
+        });
+
+    $app->helper(
+        requested_fields => sub {
+            my ($c) = @_;
+
+            my $params_ref = $c->tx->req->query_params->to_hash;
+            unless (%$params_ref) {
+                return {};
+            }
+
+            my %fields = map {
+                my $orig = $_;
+                $orig =~ m/\[(\w+)\]/;
+                ($1 => [split(',', $params_ref->{$_})])
+            } grep {
+                $_ =~ m/^fields\[/
+            } keys(%$params_ref);
+
+            my $path          = $c->tx->req->url->path;
+            my $main_resource = $path->parts->[0];
+            if ($namespace) {
+                my $idx = length(split('/', $namespace)) - 1;
+                $main_resource = $path->parts->[$idx + 1];
+            }
+
+            if ($main_resource) {
+                return {
+                    fields => delete($fields{$main_resource}),
+                    %fields ? (related_fields => \%fields) : (),
+                };
+            }
+
+            return { related_fields => \%fields };
+        });
 }
 
 1;
@@ -182,7 +209,7 @@ Mojolicious::Plugin::JSONAPI - Mojolicious Plugin for building JSON API complian
 
 =head1 VERSION
 
-version 1.3
+version 1.4
 
 =head1 SYNOPSIS
 
@@ -341,9 +368,35 @@ to the response as-is. Use C<resource_document> to generate the right structure
 Convenience helper for controllers. Takes the query param C<include>, used to indicate what relationships to include in the
 response, and splits it by ',' to return an ArrayRef.
 
- # GET /api/posts?include=comments,author
+ GET /api/posts?include=comments,author
  my $include = $c->requested_resources(); # ['comments', 'author']
 
+=head2 requested_fields
+
+Takes each query param C<fields[TYPE]> and creates a HashRef containing all its requested fields along with
+any relationship fields. This is useful if you only want to return a subset of attributes for a resource.
+
+The HashRef produced is suitable to pass directly to the options of C<JSONAPI::Document::resource_document>.
+
+Included fields should be direct attributes of the resource, not its relationships. See C<requested_resources>
+for that use case.
+
+The main resource should be in the plural form inside the param (i.e. 'posts', not 'post'), and related resources
+in their correct form.
+
+ GET /api/posts?fields[posts]=slug,title&fields[comments]=likes&fields[author]=name,email
+
+ my $fields = $c->requested_fields();
+
+ # Out:
+ {
+    fields => ['slug', 'title'],
+    related_fields => {
+        comments => ['likes'],
+        author => ['name', 'email']
+    }
+ }
+
 =head2 resource_document
 
 Available in controllers:
